@@ -6,13 +6,11 @@ Common Support for diffractometers
 
 .. autosummary::
 
-    ~Constraint
     ~Diffractometer
 
 """
 
 
-import collections
 from ophyd import Component as Cpt
 from ophyd import PositionerBase
 from ophyd import PseudoPositioner
@@ -25,18 +23,16 @@ import pint
 import pyRestTable
 
 from . import calc
+from . import __version__
+from .util import Constraint
 
 
 __all__ = """
-    Constraint
     Diffractometer
 """.split()
 logger = logging.getLogger(__name__)
 
 
-Constraint = collections.namedtuple("Constraint", ("low_limit", "high_limit", "value", "fit"))
-
-
 class Diffractometer(PseudoPositioner):
     """Diffractometer pseudopositioner
 
@@ -53,10 +49,16 @@ class Diffractometer(PseudoPositioner):
         ~undo_last_constraints
         ~pa
         ~wh
+        ~_calc_energy_update_permitted
+        ~_constraints_dict
+        ~_constraints_for_databroker
         ~_energy_changed
+        ~_energy_offset_changed
+        ~_energy_units_changed
+        ~_push_current_constraints
+        ~_set_constraints
         ~_update_calc_energy
 
-
     This has a corresponding calculation engine from **hklpy** that does
     forward and inverse calculations.
 
@@ -77,6 +79,8 @@ class Diffractometer(PseudoPositioner):
         The decision function to use when multiple solutions exist for a given
         forward calculation. Defaults to arbitrarily picking the first
         solution.
+    engine : str, optional
+        Calculation engine name.  Default: ``hkl``
     read_attrs : list, optional
         Read attributes default to all pseudo and real positioners
     configuration_attrs : list, optional
@@ -93,6 +97,10 @@ class Diffractometer(PseudoPositioner):
         (as used in `hkl.diffract.Diffractometer`), `calc_inst` must be
         specified upon initialization.
 
+    max_forward_iterations : ``ophyd.Signal`` (default value: 100)
+        Maximum allowed number of iterations in the ``forward()``
+        method before failing to find a forward() solution.
+
     See Also
     --------
     :class:`~hkl.geometries.E4CH`
@@ -108,7 +116,6 @@ class Diffractometer(PseudoPositioner):
     :class:`~hkl.geometries.SoleilSixs`
     :class:`~hkl.geometries.SoleilSixsMed1p2`
     :class:`~hkl.geometries.SoleilSixsMed2p2`
-    :class:`~hkl.geometries.TwoC`
     :class:`~hkl.geometries.Zaxis`
     """
 
@@ -181,35 +188,104 @@ class Diffractometer(PseudoPositioner):
         attr="calc.sample.uz.value",
         doc="uz portion of the U matrix",
     )
+
+    diffractometer_name = Cpt(
+        AttributeSignal,
+        attr="name",
+        doc="Diffractometer name",
+        write_access=False,
+    )
+    _hklpy_version_ = __version__
+    _hklpy_version = Cpt(
+        AttributeSignal,
+        attr="_hklpy_version_",
+        doc="hklpy version",
+        write_access=False,
+    )
+    _pseudos = Cpt(
+        AttributeSignal,
+        attr="PseudoPosition._fields",
+        doc="Pseudo Positioners",
+        write_access=False,
+    )
+    _reals = Cpt(
+        AttributeSignal,
+        attr="RealPosition._fields",
+        doc="Real Positioners",
+        write_access=False,
+    )
+    _constraints = Cpt(
+        ArrayAttributeSignal,
+        attr="_constraints_for_databroker",
+        doc="Constraints",
+        write_access=False,
+    )
+    _mode = Cpt(
+        AttributeSignal,
+        attr="calc.engine.mode",
+        doc="Mode of Operation",
+        write_access=False,
+    )
+    orientation_attrs = Cpt(
+        AttributeSignal,
+        attr="_orientation_attrs",
+        doc="Orientation Attributes",
+        write_access=False,
+    )
+
+    _orientation_attrs = """
+    orientation_attrs
+    geometry_name class_name
+    UB U ux uy uz
+    energy energy_units energy_offset
+    sample_name lattice lattice_reciprocal reflections_details
+    _pseudos _reals
+    _constraints _mode
+    diffractometer_name _hklpy_version
+    """.split()
+
+    max_forward_iterations = Cpt(Signal, value=100, kind="config")
     # fmt: on
 
+    # fmt: off
     def __init__(
         self,
         prefix,
         calc_kw=None,
         decision_fcn=None,
         calc_inst=None,
+        engine="hkl",
         *,
         configuration_attrs=None,
         read_attrs=None,
         **kwargs,
     ):
+        # fmt: on
         if calc_inst is not None:
             if not isinstance(calc_inst, self.calc_class):
-                raise ValueError("Calculation instance must be derived  from the class {self.calc_class}")
+                # fmt: off
+                raise ValueError(
+                    "Calculation instance must be derived"
+                    f" from the class {self.calc_class}"
+                )
+            # fmt: on
             self._calc = calc_inst
 
         else:
             if calc_kw is None:
                 calc_kw = {}
 
-            self._calc = self.calc_class(lock_engine=True, **calc_kw)
+            self._calc = self.calc_class(engine=engine, lock_engine=True, **calc_kw)
 
         if not self.calc.engine_locked:
             # Reason for this is that the engine determines the pseudomotor
             # names, so if the engine is switched from underneath, the
             # pseudomotor will no longer function properly
-            raise ValueError("Calculation engine must be locked  (CalcDiff.lock_engine set)")
+            # fmt: off
+            raise ValueError(
+                "Calculation engine must be locked (CalcDiff.lock_engine set)"
+            )
+            # fmt: on
 
         if configuration_attrs is None:
             configuration_attrs = """
@@ -231,6 +307,8 @@ class Diffractometer(PseudoPositioner):
             # fmt: on
         )
 
+        # write the crystal orientation information in descriptor doc
+        self.configuration_attrs += self._orientation_attrs
         self._constraints_stack = []
 
         if read_attrs is None:
@@ -360,12 +438,22 @@ class Diffractometer(PseudoPositioner):
 
     @pseudo_position_argument
     def forward(self, pseudo):
-        solutions = self.calc.forward_iter(start=self.position, end=pseudo, max_iters=100)
+        """
+        Calculate the real positions given the pseudo positions (hkl -> angles).
+
+        Return the default solution using the ``_decision_fcn()``.
+        """
+        solutions = self.calc.forward_iter(
+            start=self.position, end=pseudo, max_iters=self.max_forward_iterations.get()
+        )
         logger.debug("pseudo to real: %s", solutions)
         return self._decision_fcn(pseudo, solutions)
 
     @real_position_argument
     def inverse(self, real):
+        """
+        Calculate the pseudo positions given the real positions (angles -> hkl).
+        """
         self.calc.physical_positions = real
         return self.PseudoPosition(*self.calc.pseudo_positions)
 
@@ -409,21 +497,44 @@ class Diffractometer(PseudoPositioner):
             self._set_constraints(self._constraints_stack[0])
             self._constraints_stack = []
 
+    @property
+    def _constraints_dict(self):
+        """Return the constraints."""
+        return {
+            # fmt:off
+            m: Constraint(
+                *self.calc[m].limits,
+                self.calc[m].value,
+                self.calc[m].fit,
+            )
+            # fmt:on
+            for m in self.RealPosition._fields
+        }
+
+    @property
+    def _constraints_for_databroker(self):
+        """
+        Return the constraints for databroker.
+
+        Cannot write a dictionary from bluesky because all values must
+        be of the same data type, chosen from the first item in the
+        list. Just write the values (the boolean will be written as a
+        float.) The constraints will be written in the order of the real
+        positioners.
+        """
+        # fmt: off
+        return [
+            tuple(self._constraints_dict[p]._asdict().values())
+            for p in self.RealPosition._fields
+        ]
+        # fmt: on
+
     def show_constraints(self, fmt="simple", printing=True):
         """Print the current constraints in a table."""
         tbl = pyRestTable.Table()
         tbl.labels = "axis low_limit high_limit value fit".split()
-        for m in self.real_positioners._fields:
-            # fmt: off
-            tbl.addRow(
-                (
-                    m,
-                    *self.calc[m].limits,
-                    self.calc[m].value,
-                    self.calc[m].fit,
-                )
-            )
-            # fmt: on
+        for k, c in self._constraints_dict.items():
+            tbl.addRow((k, *c))
 
         if printing:
             print(tbl.reST(fmt=fmt))
@@ -442,17 +553,16 @@ class Diffractometer(PseudoPositioner):
         constraints = {
             m: Constraint(*self.calc[m].limits, self.calc[m].value, self.calc[m].fit)
             for m in self.real_positioners._fields
-            # TODO: any other positioner constraints
         }
         self._constraints_stack.append(constraints)
 
     def _set_constraints(self, constraints):
         """set diffractometer's constraints"""
         for axis, constraint in constraints.items():
-            self.calc[axis].limits = (
-                constraint.low_limit,
-                constraint.high_limit,
-            )
+            self.calc[axis].limits = list(constraint)[0:2]
+            #     constraint.low_limit,
+            #     constraint.high_limit,
+            # )
             self.calc[axis].value = constraint.value
             self.calc[axis].fit = constraint.fit
 
@@ -501,7 +611,7 @@ class Diffractometer(PseudoPositioner):
 
         EXAMPLE::
 
-            In [1]: from hkl.geometries import SimulatedK4CV
+            In [1]: from hkl import SimulatedK4CV
 
             In [2]: k4cv = SimulatedK4CV('', name='k4cv')
 
@@ -646,7 +756,7 @@ class Diffractometer(PseudoPositioner):
 
         EXAMPLE::
 
-            In [1]: from hkl.geometries import SimulatedK4CV
+            In [1]: from hkl import SimulatedK4CV
 
             In [2]: k4cv = SimulatedK4CV('', name='k4cv')
 
