@@ -1,15 +1,11 @@
-import gi
+from hkl import SimulatedE4CV
+from hkl.calc import A_KEV
+from hkl.diffract import Constraint
 import numpy.testing
 import pint
 import pyRestTable
 import pytest
 
-gi.require_version("Hkl", "5.0")
-# NOTE: MUST call gi.require_version() BEFORE import hkl
-from hkl.calc import A_KEV
-from hkl.diffract import Constraint
-from hkl import SimulatedE4CV
-
 
 class Fourc(SimulatedE4CV):
     ...
